import * as React from "react";
import { IObservableArrayEventArgs, ObservableArrayAction } from '../../Core/Observable';
import { IFocusZoneProps } from '../../FocusZone';
import { IIconProps } from '../../Icon';
import { IIntersectionContext } from '../../Intersection';
import { ILinkProps } from '../../Link';
import { IBehavior } from '../../Utilities/Behavior';
import { IEventDispatch } from '../../Utilities/Dispatch';
import { BoltDragEvent } from '../../Utilities/DragDrop';
import { IIndexed, ISelection, ISelectOptions, ISelectableUI, ISelectionRange } from '../../Utilities/Selection';
import { IItemProvider } from '../../Utilities/Provider';
/**
 * The IListCellDetails is used to describe the elements and indexes for a given cell.
 * The header row uses rowIndex = -1.
 */
export interface IListCellDetails {
    cellElement: HTMLElement | null;
    cellIndex: number;
    rowElement: HTMLElement | null;
    rowIndex: number;
}

/**
 * An IListRow is used communicate details about a given row in a list.
 */
export interface IListRow<T> extends IIndexed {
    /**
     * Data that represents the instance of T for this row. In some cases the
     * row data may not be available.
     */
    data: T;
}

/**
 * The List used an IListSelection to manage the selection state for the
 * rows within the list. The is required for a multi-select list and optional
 * for single select lists.
 */
export interface IListSelection extends ISelection, IBehavior<{}, ISelectableUI> {}

/**
 * Options to describe the list selection behavior.
 */
export interface IListSelectionOptions extends ISelectOptions {
    /**
     * Set to true to select items when they receive focus.
     */
    selectOnFocus?: boolean;
}

export interface IOverlayRenderProps {
    rowElement: HTMLElement;
}

export interface IRowOverlayableUI {
    /**
     * addOverlay is used to create, or update an existing, overlay over an existing list row.
     * This allows the caller to apply effects to the row by having elements
     * that appear over top of the row.
     *
     * @param id This is a unique id for this overlay.
     * @param rowIndex The index of the row the overlay should hover over.
     * @param render A function which returns the React nodes that should be placed within the overlay.
     *  The parent element will be positioned, and have the same height and width
     *  as the underlying row element.
     * @param zIndex If there are multiple overlays the caller can use the zIndex
     *  parameter to ensure they are layered correctly.
     */
    addOverlay(id: string, rowIndex: number, render: (props: IOverlayRenderProps) => React.ReactNode, zIndex?: number): void;

    /**
     * removeOverlay is used to remove an overlay that has already been registered.
     * The unique id is all that is needed.
     * @param id This is a unique id for this overlay.
     */
    removeOverlay(id: string): void;
}

export interface IDragDroppableUI extends ISelectableUI, IRowOverlayableUI {}

export interface BoltListDragEvent<T, D> extends BoltDragEvent<T, D, IListDragDropSecondaryData> {}

export interface IListDragDropSecondaryData {
    readonly index: number;
    readonly sourceId: string;
}

/**
 * IList<T> is the interace the List component exposes for use by callers.
 * If they ref the component they should access the List with this interface.
 */
export interface IList<T> extends ISelectableUI, IRowOverlayableUI {}

/**
 * An interface to group properties that get passed to the row renderer of each list row.
 */
export interface IListItemDetails<T> {
    /**
     * Sets aria-busy on the list item element.
     */
    ariaBusy?: boolean;

    /**
     * ariaLabel allows the caller to describe the elements contents to assistive
     * technology.
     */
    ariaLabel?: string;

    /**
     * Sets aria-posinset on the tr element.  Defaults to the item index.
     */
    ariaPosInSet?: number | null;

    /**
     * Sets aria-setsize on the tr element.  Defaults to the itemProvider length.
     */
    ariaSetSize?: number | null;

    /**
     * The data represents the object being rendered in this row. If the caller
     * has asynchronous loading of rows, the data MAY be undefined while we wait
     * for the data to be resolved.
     */
    data?: T;

    /**
     * An event dispatch the row MAY use to dispatch custom events to list behaviors.
     */
    eventDispatch: IEventDispatch;

    /**
     * Set to true to exclude the list item from the focus zone.
     */
    excludeFocusZone?: boolean;

    /**
     * If false, the row will get tabIndex=0, otherwise tabIndex=-1.
     * @default true
     */
    excludeTabStop?: boolean;

    /**
     * Properties used to render the list as a whole.
     */
    listProps: IListProps<T>;

    /**
     * The row MUST call onFocusItem when a row or one of the child elements of the row receives focus.
     */
    onFocusItem: (rowIndex: number, event: React.FocusEvent<HTMLElement>) => void;
}

/**
 * IListProps are used to describe the set of features used by the List component
 * and any attached behaviors.
 *
 * The list provides NO scrolling or virutalization behaviors by default. It requires
 * the user to define the behavior through an Intersection component. This is used to
 * monitor the visiblity with the intersections viewport.
 *
 * If the user just wants a List where its contents scroll, make sure you use a
 * ScrollableList or SimpleList with scrollable set to true, instead of a List. If
 * the user wants to embed the list in scrollable element with other elements the
 * scrolling element should have an Intersection defined around it.
 */
export interface IListProps<T> {
    /**
     * The caller MUST supply the set of items to be shown through the ItemProvider.
     * The IItemProvider allows the caller to store their items in the form that
     * bests suits their needs but gives the list a well defined interface for
     * requesting the items. This can include async fetching of items through
     * observables.
     *
     * There is simple ArrayItemProvider<T> for those that just have a set of items
     * they want to supply without writing a custom ItemProvider.
     */
    itemProvider: IItemProvider<T>;

    /**
     * Behaviors can be added to the list and monitor events and interact with the
     * component through the IList methods.
     */
    behaviors?: IBehavior<Partial<IListProps<T>>, Partial<IList<T>>>[];

    /**
     * css className to add to the list root element.
     */
    className?: string;

    /**
     * Number of columns the list is going to represent. This is important to get
     * correct if the contents are going to show as multi-column since the
     * virtualization is emitting elements (non-visible) into the list for paging.
     *
     * By default the list is a single column value.
     */
    columns?: number;

    /**
     * The index of the row that should be tabbable before the list has received focus.
     * @default 0
     */
    defaultTabbableRow?: number;

    /**
     * The caller can supply an EventDispatch to the list if it wishes to
     * participate it extending the behaviors. If one isnt supplied the list will
     * create its own dispatcher when behviors are supplied.
     */
    eventDispatch?: IEventDispatch;

    /**
     * focuszoneProps allows the caller to manage the how the list rows are focused.
     * The default focuszone if one isnt supplied is a Vertical non-cyclic focus zone.
     */
    focuszoneProps?: IFocusZoneProps | null;

    /**
     * Unique Id for this list.
     */
    id?: string;

    /**
     * initialPageCount is used when the list is drawing pages and no pages are
     * visible. This primarily happens when the list is initially rendered.
     *
     * The impact of setting this value effects performance of the initial render.
     * Since the component performs 0 measurements it renders the initialPages and
     * uses the IntersectionObserver to determine whether or not it has filled the
     * components viewport. This means that in the first pass it will render the
     * requested rows and then update the rows after the observer reports on the
     * visibility of the rendered pages. If 3 was not enough to fill the UI it will
     * render more, but this will happen async.
     *
     * The default is 3 pages with a default of 10 rows per page - 30 rows..
     */
    initialPageCount?: number;

    /**
     * maxWidth is used to control the maximum width of the list.
     */
    maxWidth?: string;

    /**
     * minWidth is used to control the minumum width of the list.
     */
    minWidth?: string;

    /**
     * onActivate is called when the row is activated. Activation occurs on
     * the Enter keystroke or double click.
     *
     * @param event This is the event that is causing the activation.
     * @param listRow Details about the list row being activated.
     */
    onActivate?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<T>) => void;

    /**
     * onSelect is called when the row is selected. Selection occurs on the
     * Space keystorke or click.
     *
     * @param event - This is the event that is causing the selection.
     * @param listRow - Details about the list row being selected.
     */
    onSelect?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<T>) => void;

    /**
     * pageSize controls the granularity of row rendering. The list always renders
     * a full page worth of rows even when they are not needed to fill the viewport.
     *
     * Smaller values will help reduce the number of wasted rows that are rendered
     * outside the viewport, but will force the list to re-render more often as
     * scrolling occurs.
     */
    pageSize?: number;

    /**
     * When a row's value is given as an ObservableValue with an undefined value,
     * the list will render a Loading row for the content. The default will be
     * a shimmer row that is semi random and matches the content.
     *
     * @param index This is the 0 based row index that should be rendered.
     * @param details Additional details about this row.
     */
    renderLoadingRow?: (rowIndex: number, details: IListItemDetails<T>) => JSX.Element;

    /**
     * All callers must supply a function for rendering the list row.
     *
     * The requirements of this function are quite complicated and before writing
     * a new row renderer you should ensure one doesnt already exist that solves
     * your needs.
     *
     * Requirements:
     *
     * 1) The row function MUST return a singlely rooted component that resolves
     * to single rooted element, or return a single element that is either a <tr>
     * or another acceptable tag that is marked as a display: table-row.
     *
     * 2) The row function MUST only include direct child elements that are either
     * <td>'s or acceptable elements that are maked as display: table-cell. The
     * renderer MUST return one element for each defined column even if there is
     * no data to be rendered in the column.
     *
     * 3) The row function MUST call onFocusItem when a either the row element of
     * any of the rows child elements receive the focus. This is needed to ensure
     * navigation within the list as well as in and out of the list function
     * correctly.
     *
     * 4) The row function MUST add data-row-index on the row element.  This is
     * needed to ensure mouse events can be translated to rows.
     *
     * 5) The row function MAY dispatch events for behaviors but is not required.
     * If any events are dispatched they should be documented on the row renderer.
     *
     * 6) The row function is responsible for all accessibility and focus
     * management within the row.
     *
     * @param index This is the 0 based row index that should be rendered.
     * @param item This is the object that represents the current rows data.
     * @param details Additional details to handle aria attributes and focus.
     */
    renderRow: (rowIndex: number, item: T, details: IListItemDetails<T>) => JSX.Element;

    /**
     * Lists MAY render a header above the list contents. This is done through the
     * onRenderHeader. Like the renderRow method, there are set of restrictions the
     * implmentation MUST follow.
     *
     * Requirements:
     *
     * 1) The header function MUST return a singlely rooted component that resolves
     * to single rooted element, or return a single element that is either a <tr>
     * or another acceptable tag that is marked as a display: table-row.
     *
     * 2) The header function MUST only include direct child elements that are either
     * <td>'s or acceptable elements that are maked as display: table-cell. The
     * renderer MUST return one element for each defined column even if there is
     * no data to be rendered in the column.
     */
    renderHeader?: () => JSX.Element;

    /**
     * role defines the aria role of the list and defaults to "listbox"
     */
    role?: string;

    /**
     * If the caller has variable height rows they can specify the rowHeight they
     * want used to estimate the size of virtualized rows. This means that when rows
     * are not rendered, the component will create virtual space for those rows to
     * ensure the scrolling behavior acts appropriately.
     *
     * If the list has fixed size rows there is no need to specify a rowHeight. The
     * list will determine the height of the rows after the initial render when
     * the observer reports on page visibility.
     *
     * Question: How do I determine the rowHeight if their are variable height rows.
     * This one is a tough question, and the general answer is come up with a fair
     * average for the rows on a given page. If the select too large or too small
     * scrolling behaviors can become a bit odd, generally select on the smaller
     * side if you are unsure.
     */
    rowHeight?: number;

    /**
     * A selection object can be supplied for managing the list selection. This
     * is not required since the list offers onSelect as a delegate. If the caller
     * wants multi-selction they must use an IListSelection that supports multi
     * select.
     *
     * There is a basic ListSelection implementation available from the List
     * component.
     */
    selection?: IListSelection;

    /**
     * Using singleClickActivation will activate the item when the row is clicked.
     * Where setting singleClickActivation to false will require a doubleclick to
     * activate a given row.
     *
     * @default true
     */
    singleClickActivation?: boolean;

    /**
     * width can be any supported css width value, either a %, px, and vw value.
     */
    width?: string;
}

/**
 * ISimpleListCell instances are used with the SimpleList to render basic cells
 * in a list or table. It supports an optional icon, text and the text can be made
 * into a hyper link.
 */
export interface ISimpleListCell {
    /**
     * iconProps for the icon within the cell.
     */
    iconProps?: IIconProps;

    /**
     * text that should within the cell.
     */
    text?: string;

    /**
     * Optionally className that should be applied to the text.
     */
    textClassName?: string;

    /**
     * A link that should be applied to the text of the cell.
     */
    href?: string;
}

/**
 * ISimpleListProps are used to create a SimpleList. The SimpleList renders its
 * rows using either an ISimpleCell, string, or number.
 */
export interface ISimpleListProps {
    /**
     * Behaviors can be added to the list and monitor events and interact with the
     * component through the IList methods.
     */
    behaviors?: Array<IBehavior<Partial<IListProps<ISimpleListCell | string | number>>, Partial<IList<ISimpleListCell | string | number>>>>;

    /**
     * css className to add to the list element.
     */
    className?: string;

    /**
     * The caller can supply an EventDispatch to the list if it wishes to
     * participate it extending the behaviors. If one isnt supplied the list will
     * create its own dispatcher when behviors are supplied.
     */
    eventDispatch?: IEventDispatch;

    /**
     * focuszoneProps allows the caller to manage the how the list rows are focused.
     * The default focuszone if one isnt supplied is a Vertical non-cyclic focus zone.
     */
    focuszoneProps?: IFocusZoneProps;

    /**
     * Unique Id for this list.
     */
    id?: string;

    /**
     * initialPageCount is used when the list is drawing pages and no pages are
     * visible. This primarily happens when the list is initially rendered.
     *
     * The impact of setting this value effects performance of the initial render.
     * Since the component performs 0 measurements it renders the initialPages and
     * uses the IntersectionObserver to determine whether or not it has filled the
     * components viewport. This means that in the first pass it will render the
     * requested rows and then update the rows after the observer reports on the
     * visibility of the rendered pages. If 3 was not enough to fill the UI it will
     * render more, but this will happen async.
     *
     * The default is 3 pages with a default of 10 rows per page - 30 rows..
     */
    initialPageCount?: number;

    /**
     * List MUST supply the set of items it wants to show through the ItemProvider.
     * The IItemProvider allows the caller to store their items in the form that
     * bests suits their needs but gives the list a well defined interface for
     * requesting the items. This can include async fetching of items through
     * observables.
     *
     * There is simple ArrayItemProvider<T> for those that just have a set of items
     * they want to supply without writing a custom ItemProvider.
     */
    itemProvider: IItemProvider<ISimpleListCell | string | number>;

    /**
     * onActivate is called when the row is activated. Activation occurs on
     * the Enter keystroke or double click.
     */
    onActivate?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<ISimpleListCell | string | number>) => void;

    /**
     * onSelect is called when the row is selected. Selection occurs on the
     * Space keystorke or click.
     */
    onSelect?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<ISimpleListCell | string | number>) => void;

    /**
     * A selection object can be supplied for managing the list selection. This
     * is not required since the list offers onSelect as a delegate. If the caller
     * wants multi-selction they must use an IListSelection that supports multi
     * select.
     *
     * There is a basic ListSelection implementation available from the List
     * component.
     */
    selection?: IListSelection;

    /**
     * scrollable should be set to true if the list is not contained in a
     * scrolling element. This will ensure the list scrolls vertically within
     * the list element itself.
     */
    scrollable?: true;

    /**
     * width can be any supported css width value, either a %, px, and vw value.
     */
    width?: string;
}

/**
 * If the caller wants to control the row they can render the entire row on their
 * own, or they can use a ListItem to get the majority of the row handled.
 *
 * Using the ListItem instead of renderListItem gives the caller access to more
 * detailed control over the list item. An example is the ability to render the
 * list item as a link (a tag) instead of a table row.
 */
export interface IListItemProps<T> {
    /**
     * css class that will be added to the list item.
     */
    className?: string;

    /**
     * Details about the list item being rendered. This is given to the caller
     * when a row needs to be rendered.
     */
    details: IListItemDetails<T>;

    /**
     * Index of the row that is being rendered.
     */
    index: number;

    /**
     * If the list item should be rendered as a link, the caller can supply the
     * links properties. These are merged with the lists properties to build
     * a list row that is a anchor instead of a table row.
     */
    linkProps?: ILinkProps;
}