import * as tslib_1 from "tslib";
import "../../CommonImports";
import "../../Core/core.css";
import "../../Common.css";
import "./List.css";
import "./ListDropIndicator.css";
import * as React from "react";
import { ObservableArray, ObservableLike } from '../../Core/Observable';
import { FocusWithin } from '../../FocusWithin';
import { FocusZone, FocusZoneContext, FocusZoneDirection } from '../../FocusZone';
import { Icon } from '../../Icon';
import { Intersection, IntersectionContext } from '../../Intersection';
import { Link } from '../../Link';
import { Observer } from '../../Observer';
import { Tooltip } from '../../TooltipEx';
import { css, eventTargetContainsNode, getSafeId, KeyCode } from '../../Util';
import { EventDispatch } from '../../Utilities/Dispatch';
/**
 * The List component is used to render a collection of items with a series of rows.
 */
var List = /** @class */ (function (_super) {
    tslib_1.__extends(List, _super);
    function List(props) {
        var _this = _super.call(this, props) || this;
        // Track the table element used to render the rows.
        _this.listElement = React.createRef();
        _this.bodyElement = React.createRef();
        // Manage data about pages, including their spacers.
        _this.spacerElements = {};
        // Track the row with focus.
        _this.focusIndex = -1;
        _this.onDispatch = function (event) {
            _this.state.eventDispatch.dispatchEvent(event);
        };
        _this.onClick = function (event) {
            _this.onDispatch(event);
            if (!event.defaultPrevented) {
                if (_this.props.singleClickActivation && _this.listElement.current && !eventTargetContainsNode(event, ["A"], _this.listElement.current)) {
                    var rowIndex = cellFromEvent(event).rowIndex;
                    var item = ObservableLike.getValue(_this.state.rows[rowIndex]);
                    if (rowIndex >= 0 && item) {
                        _this.rowActivated(event, { data: item, index: rowIndex });
                    }
                }
            }
        };
        _this.onDoubleClick = function (event) {
            _this.onDispatch(event);
            if (!event.defaultPrevented) {
                var rowIndex = cellFromEvent(event).rowIndex;
                var item = ObservableLike.getValue(_this.state.rows[rowIndex]);
                if (rowIndex >= 0 && item) {
                    _this.rowActivated(event, { data: item, index: rowIndex });
                }
            }
        };
        _this.onFocusItem = function (rowIndex, event) {
            var focusIndex = _this.focusIndex;
            if (focusIndex !== rowIndex) {
                // We are going to pass the event along after state is updated, we need to ensure
                // the event doesnt get re-used.
                event.persist();
                // We need to re-render the previously focused row and newly focused row so we will
                // clear the cached values.
                if (focusIndex >= 0) {
                    delete _this.state.renderedRows[focusIndex];
                }
                else if (_this.props.defaultTabbableRow !== undefined) {
                    // If there was a tabble row that was not the focusIndex row we need to update this
                    // row as well to get it re-rendered without the tabIndex.
                    delete _this.state.renderedRows[_this.props.defaultTabbableRow];
                }
                delete _this.state.renderedRows[rowIndex];
                _this.focusIndex = rowIndex;
                var item = ObservableLike.getValue(_this.state.rows[rowIndex]);
                if (item) {
                    _this.rowSelected(event, { data: item, index: rowIndex });
                }
            }
        };
        _this.onKeyDown = function (event) {
            _this.onDispatch(event);
            if (!event.defaultPrevented) {
                var focusIndex = _this.focusIndex;
                if (event.which === KeyCode.enter) {
                    if (focusIndex >= 0 && !eventTargetContainsNode(event, ["A"])) {
                        var item = ObservableLike.getValue(_this.state.rows[focusIndex]);
                        if (item) {
                            _this.rowActivated(event, { data: item, index: focusIndex });
                        }
                    }
                }
                else if (event.which === KeyCode.space) {
                    event.preventDefault();
                }
            }
        };
        _this.onIntersect = function () {
            var _a = _this.state, pageSize = _a.pageSize, rowCount = _a.rowCount;
            var _b = _this.state, firstRow = _b.firstRow, lastRow = _b.lastRow, rowHeight = _b.rowHeight;
            // console.log({ phase: "onIntersect - Start", firstRow, lastRow, rowHeight });
            // Ignore events if we dont have a our basic elements resolved (this should never happen).
            if (!_this.listElement.current || !_this.bodyElement.current) {
                return;
            }
            // We are going to enumerate all the children, if the row is in the viewport
            // we will determine if it should be paged out.
            var rowElements = _this.bodyElement.current.children;
            // If a rowHeight was specified we will compute on based on the average rowHeight in the
            // first page rendered.
            if (rowHeight === 0) {
                if (rowElements.length > 0) {
                    var totalHeight = 0;
                    var childCount = 0;
                    // Loop through all children and average the rowHeight's.
                    for (var childIndex = 0; childIndex < rowElements.length; childIndex++) {
                        var childHeight = _this.bodyElement.current.children[childIndex].getBoundingClientRect().height;
                        if (childHeight > 0) {
                            totalHeight += childHeight;
                            childCount++;
                        }
                    }
                    // Make sure we have at least one child row that has size.
                    if (childCount > 0) {
                        rowHeight = Math.ceil(totalHeight / childCount);
                    }
                }
                if (rowHeight === 0) {
                    return;
                }
                // console.log({ phase: "onIntersect - Compute RowHeight", rowHeight });
            }
            // Determine the location of the intersection within the page. This is the element
            // we are scrolling within.
            var intersectionRect = _this.intersectionContext.root.getBoundingClientRect();
            var rootMargin = _this.intersectionContext.rootMargin;
            // Track the first and last row elements for adjusting the range.
            var firstRowElement;
            var lastRowElement;
            var firstRowUpdated = firstRow;
            var lastRowUpdated = lastRow;
            // Go through the viewport pages and determine if any are out of range and should be
            // paged out. Range is defined as more than 1 page of estimated rows away from the
            // nearest edge. If you dont allow for 1 page of estimated rows it may thrash pages
            // in and out of materialization.
            for (var childIndex = 0; childIndex < rowElements.length; childIndex++) {
                // Determine if this child is in the viewport, ignore rows that are not.
                var rowElement = rowElements[childIndex];
                var rowIndex = getAttributeAsNumber(rowElement, "data-row-index");
                if (rowIndex >= firstRow && rowIndex <= lastRow) {
                    var rowRect = rowElement.getBoundingClientRect();
                    if (rowRect.bottom < intersectionRect.top - rootMargin - rowHeight * 2) {
                        firstRowUpdated++;
                    }
                    else if (rowRect.top > intersectionRect.bottom + rootMargin + rowHeight * 2) {
                        lastRowUpdated--;
                    }
                    // We will save the first and last rows for later computations.
                    if (rowIndex === firstRow) {
                        firstRowElement = rowElement;
                    }
                    if (rowIndex === lastRow) {
                        lastRowElement = rowElement;
                    }
                }
            }
            // If the page range is inverted then all pages have been hidden and we should
            // recompute the viewport based on the scrollTop of our intersection and intersection height.
            if (firstRowUpdated > lastRowUpdated) {
                var scrollTop = _this.intersectionContext.root.scrollTop - _this.listElement.current.offsetTop;
                firstRowUpdated = Math.max(0, Math.min(rowCount, Math.floor(scrollTop / rowHeight)));
                lastRowUpdated = Math.min(rowCount, firstRowUpdated + Math.ceil(intersectionRect.height / rowHeight));
            }
            else {
                // If the firstPage didnt move down, we may need more pages above.
                if (firstRowUpdated === firstRow && firstRowElement) {
                    var rowRect = firstRowElement.getBoundingClientRect();
                    var availableSpace = rowRect.top - (intersectionRect.top - rootMargin - rowHeight);
                    if (availableSpace > 0) {
                        firstRowUpdated -= Math.ceil(availableSpace / rowHeight);
                    }
                }
                // If the lastPage didnt move up, we may need more pages below.
                if (lastRowUpdated === lastRow && lastRowElement) {
                    var rowRect = lastRowElement.getBoundingClientRect();
                    var availableSpace = intersectionRect.bottom + rootMargin + rowHeight - rowRect.bottom;
                    if (availableSpace > 0) {
                        lastRowUpdated += Math.ceil(availableSpace / rowHeight);
                    }
                }
            }
            // Make sure our page boundary stays in the available page range.
            // Add one page to either end of the viewport range.
            firstRowUpdated = Math.max(firstRowUpdated - pageSize, 0);
            lastRowUpdated = Math.min(lastRowUpdated + pageSize, rowCount - 1);
            // console.log({ phase: "onIntersect - End", firstRowUpdated, lastRowUpdated, rowHeight });
            // Update our state if and only if something has changed.
            if (firstRowUpdated !== firstRow || lastRowUpdated !== lastRow || rowHeight !== _this.state.rowHeight) {
                /*
                 * @TODO: We need to unload data for pages that are no longer rendererd.
                 * This means not in the viewport or within any other rendered range.
                 */
                // console.log({ phase: "onIntersect - stateChange", firstRowUpdated, lastRowUpdated });
                _this.setState({ firstRow: firstRowUpdated, lastRow: lastRowUpdated, rowHeight: rowHeight });
            }
        };
        var rowCount = props.itemProvider.length;
        var pageSize = props.pageSize;
        _this.state = {
            columns: 1,
            eventDispatch: props.eventDispatch || new EventDispatch(),
            firstRow: 0,
            itemProvider: props.itemProvider,
            lastRow: props.initialPageCount * pageSize,
            overlays: new ObservableArray(),
            pageSize: pageSize,
            renderedRows: {},
            rowCount: rowCount,
            rowHeight: props.rowHeight || 0,
            rows: {},
            scrollTop: 0
        };
        // The selection is implemented as a behavior, we will initialize it.
        if (props.selection && props.selection.initialize) {
            props.selection.initialize(props, _this, _this.state.eventDispatch);
        }
        // Initialize the supplied behaviors.
        if (props.behaviors) {
            for (var _i = 0, _a = props.behaviors; _i < _a.length; _i++) {
                var behavior = _a[_i];
                if (behavior.initialize) {
                    behavior.initialize(props, _this, _this.state.eventDispatch);
                }
            }
        }
        return _this;
    }
    List.getDerivedStateFromProps = function (props, state) {
        var rowCount = props.itemProvider.length;
        var firstRow = Math.max(0, Math.min(state.firstRow, rowCount));
        var lastRow = Math.max(firstRow, Math.min(state.lastRow, rowCount - 1));
        // Ensure out pages and providers are appropriately computed.
        var updatedState = {
            firstRow: firstRow,
            itemProvider: props.itemProvider,
            lastRow: lastRow,
            pageSize: props.pageSize,
            rowCount: rowCount
        };
        // If there are changes to the props that affect the cached data, we need it clear it.
        if (props.itemProvider !== state.itemProvider || props.columns !== state.columns) {
            updatedState.columns = props.columns;
            updatedState.renderedRows = {};
            updatedState.rows = {};
        }
        // console.log(updatedState);
        return updatedState;
    };
    List.prototype.render = function () {
        var _this = this;
        var _a = this.props, className = _a.className, focuszoneProps = _a.focuszoneProps, id = _a.id, maxWidth = _a.maxWidth, minWidth = _a.minWidth, role = _a.role, width = _a.width;
        var _b = this.state, firstRow = _b.firstRow, lastRow = _b.lastRow, rowCount = _b.rowCount;
        var focusIndex = this.focusIndex;
        var rows = [];
        // Number of pages each spacer takes up. There are potentially two spacers above
        // or below the view port. They surround the focus range when the focus range is
        // not within the viewport.
        var topSpacer1 = 0;
        var topSpacer2 = firstRow;
        var bottomSpacer2 = Math.max(0, rowCount - lastRow - 1);
        var bottomSpacer1 = 0;
        var firstFocusRow = Number.MAX_SAFE_INTEGER;
        var lastFocusRow = 0;
        // Compute the range of focus pages, these will be either before or after the pages
        // in the viewport. We need to ensure we have one row before and one row after the
        // focus row to support arrowing up and down.
        if (focusIndex !== -1) {
            firstFocusRow = Math.max(0, focusIndex - 3);
            lastFocusRow = Math.min(rowCount, focusIndex + 3);
            // Make sure we dont draw any of the pages that are in the viewport.
            if (firstFocusRow < firstRow) {
                lastFocusRow = Math.min(lastFocusRow, firstRow - 1);
                topSpacer1 = firstFocusRow;
                topSpacer2 = firstRow - lastFocusRow - 1;
            }
            else if (lastFocusRow > lastRow) {
                firstFocusRow = Math.max(firstFocusRow, lastRow + 1);
                bottomSpacer2 = firstFocusRow - lastRow - 1;
                bottomSpacer1 = Math.max(0, rowCount - lastFocusRow - 1);
            }
        }
        // console.log({ phase: "render", firstRow, lastRow, topSpacer1, topSpacer2, bottomSpacer2, bottomSpacer1 });
        return (React.createElement(IntersectionContext.Consumer, null, function (intersectionContext) {
            _this.intersectionContext = intersectionContext;
            rows.push(_this.renderSpacer("st1", topSpacer1));
            // If the focus pages are before the viewport render them up too
            // the first page but not including the first page.
            if (firstFocusRow < firstRow) {
                for (var rowIndex = firstFocusRow; rowIndex <= lastFocusRow; rowIndex++) {
                    rows.push(_this.renderRow(rowIndex));
                }
            }
            rows.push(_this.renderSpacer("st2", topSpacer2));
            // Go through each of the rendered pages and generate the child component.
            for (var rowIndex = firstRow; rowIndex <= lastRow; rowIndex++) {
                rows.push(_this.renderRow(rowIndex));
            }
            rows.push(_this.renderSpacer("sb2", bottomSpacer2));
            // If the focus pages are after the last page in the viewport render
            // them but not including the last page.
            if (lastFocusRow > lastRow) {
                for (var rowIndex = firstFocusRow; rowIndex <= lastFocusRow; rowIndex++) {
                    rows.push(_this.renderRow(rowIndex));
                }
            }
            rows.push(_this.renderSpacer("sb1", bottomSpacer1));
            var list = (React.createElement("table", { "aria-colcount": _this.props.columns, "aria-rowcount": _this.state.itemProvider.length, className: css(className, "bolt-list relative"), id: getSafeId(id), onClick: _this.onClick, onDoubleClick: _this.onDoubleClick, onDragEnd: _this.onDispatch, onDragEnter: _this.onDispatch, onDragExit: _this.onDispatch, onDragOver: _this.onDispatch, onDragStart: _this.onDispatch, onDrop: _this.onDispatch, onKeyDown: _this.onKeyDown, onKeyUp: _this.onDispatch, onMouseDown: _this.onDispatch, onTouchStart: _this.onDispatch, ref: _this.listElement, role: role || "listbox", style: { maxWidth: maxWidth, minWidth: minWidth, width: width } },
                _this.props.renderHeader && _this.props.renderHeader(),
                React.createElement("tbody", { className: "relative", ref: _this.bodyElement },
                    _this.renderOverlay(_this.listElement),
                    rows)));
            if (focuszoneProps) {
                list = (React.createElement(FocusZone, tslib_1.__assign({}, focuszoneProps, { skipHiddenCheck: true }), list));
            }
            return (React.createElement(Observer, { itemProvider: {
                    // Supply an IObservableExpression to elevate the provider change to a state
                    // update for the entire component instead of just the observer.
                    filter: function (change, action) {
                        // Notify the selection about the change to the items.
                        if (_this.props.selection) {
                            _this.props.selection.onItemsChanged(change, action);
                        }
                        // @NOTE: For now we will just wipe out the entire cache, we can do an optimized
                        // update to the cache based on the rows that changed.
                        var updatedState = {
                            renderedRows: {},
                            rows: {}
                        };
                        // If their is a well defined rowcount we will update it and the maxPage.
                        if (_this.state.rowCount !== -1) {
                            var countChange = (change.addedItems ? change.addedItems.length : 0) -
                                (change.removedItems ? change.removedItems.length : 0);
                            if (countChange) {
                                updatedState.rowCount = _this.state.rowCount + countChange;
                                updatedState.firstRow = Math.max(0, Math.min(_this.state.firstRow, updatedState.rowCount - 1));
                                updatedState.lastRow = Math.max(updatedState.firstRow, Math.min(_this.state.lastRow +
                                    (change.index >= _this.state.firstRow && change.index <= _this.state.lastRow ? countChange : 0), updatedState.rowCount - 1));
                            }
                        }
                        // console.log(updatedState);
                        _this.setState(updatedState);
                        return false;
                    },
                    observableValue: _this.props.itemProvider
                } }, function () { return list; }));
        }));
    };
    List.prototype.componentDidMount = function () {
        if (this.intersectionContext) {
            this.intersectionContext.register(this.onIntersect);
        }
    };
    List.prototype.componentWillUnmount = function () {
        if (this.intersectionContext) {
            this.intersectionContext.unregister(this.onIntersect);
        }
    };
    List.prototype.addOverlay = function (id, rowIndex, render, zIndex) {
        if (zIndex === void 0) { zIndex = 0; }
        var overlays = this.state.overlays;
        var overlayIndex = overlays.value.findIndex(function (overlay) { return overlay.id === id; });
        var rowOverlay = { render: render, id: id, rowIndex: rowIndex, zIndex: zIndex + 1 };
        // Update the overlay if it exists for that id, otherwise add it
        if (overlayIndex >= 0) {
            overlays.change(overlayIndex, rowOverlay);
        }
        else {
            overlays.push(rowOverlay);
        }
    };
    List.prototype.removeOverlay = function (id) {
        var overlays = this.state.overlays;
        var overlayIndex = overlays.value.findIndex(function (overlay) { return overlay.id === id; });
        // Remove the overlay if it exists.
        if (overlayIndex >= 0) {
            overlays.splice(overlayIndex, 1);
        }
    };
    List.prototype.getFocusIndex = function () {
        return this.focusIndex;
    };
    List.prototype.clearCache = function (pageIndex) {
        for (var rowIndex = pageIndex * this.state.pageSize, lastIndex = rowIndex + this.state.pageSize; rowIndex < lastIndex; rowIndex++) {
            delete this.state.rows[rowIndex];
            delete this.state.renderedRows[rowIndex];
        }
    };
    List.prototype.renderLoadingRow = function (rowIndex, details) {
        return (React.createElement(ListItem, { className: "bolt-list-row-loading", details: details, index: rowIndex },
            React.createElement("div", { className: "shimmer shimmer-line", style: { width: Math.random() * 80 + 20 + "%" } }, "\u00A0")));
    };
    List.prototype.renderOverlay = function (listElementRef) {
        var _this = this;
        var _a = this.state, firstRow = _a.firstRow, lastRow = _a.lastRow, overlays = _a.overlays;
        return (React.createElement(Observer, { overlays: overlays }, function (props) {
            var bodyElement = _this.bodyElement.current;
            if (props.overlays.length > 0 && bodyElement) {
                return (React.createElement("div", { className: "bolt-list-overlay-container absolute" }, props.overlays.map(function (overlay) {
                    // Make sure the row is in the rendered range of rows before starting.
                    if (overlay.rowIndex < firstRow || overlay.rowIndex > lastRow) {
                        return null;
                    }
                    // Find the row for the given rowIndex
                    var rowElement = listElementRef.current &&
                        listElementRef.current.querySelector("[data-row-index='" + overlay.rowIndex + "']");
                    // We cant render the overlay if the row is paged out since we can't determine
                    // the location of the row.
                    if (rowElement) {
                        return (React.createElement("div", { className: "bolt-list-overlay flex-row absolute", id: getSafeId(overlay.id), key: overlay.id, style: {
                                height: rowElement.offsetHeight,
                                top: rowElement.getBoundingClientRect().top - bodyElement.getBoundingClientRect().top,
                                zIndex: overlay.zIndex * 10
                            } }, overlay.render({ rowElement: rowElement })));
                    }
                    return null;
                })));
            }
            return null;
        }));
    };
    List.prototype.renderRow = function (rowIndex) {
        var _this = this;
        var itemProvider = this.props.itemProvider;
        var _a = this.state, renderedRows = _a.renderedRows, rows = _a.rows;
        var renderedRow = renderedRows[rowIndex];
        if (!renderedRow) {
            var item_1 = rows[rowIndex];
            if (!item_1) {
                if (itemProvider.getItem) {
                    item_1 = itemProvider.getItem(rowIndex);
                }
                else {
                    item_1 = itemProvider.value[rowIndex];
                }
            }
            // @TODO: If there are no more rows, we need to handle an itemProvider with -1 length.
            if (!item_1) {
                return null;
            }
            // Save the current item in the item cache.
            rows[rowIndex] = item_1;
            var selection = this.props.selection;
            var selectionObservable = void 0;
            if (selection) {
                selectionObservable = {
                    observableValue: selection,
                    filter: function (selectedRanges) {
                        for (var _i = 0, selectedRanges_1 = selectedRanges; _i < selectedRanges_1.length; _i++) {
                            var selectionRange = selectedRanges_1[_i];
                            if (rowIndex >= selectionRange.beginIndex && rowIndex <= selectionRange.endIndex) {
                                return true;
                            }
                        }
                        return false;
                    }
                };
            }
            // Render the row, save it in the cache, and add it to the current page.
            renderedRow = (React.createElement(Observer, { item: item_1, key: rowIndex, selection: selectionObservable }, function (props) {
                var _a = _this.props, renderRow = _a.renderRow, defaultTabbableRow = _a.defaultTabbableRow, renderLoadingRow = _a.renderLoadingRow;
                var focusIndex = _this.focusIndex;
                var tabbableIndex = focusIndex >= 0 ? focusIndex : defaultTabbableRow;
                var rowItem = ObservableLike.getValue(item_1);
                var itemDetails = {
                    ariaBusy: !props.item,
                    eventDispatch: _this.state.eventDispatch,
                    data: rowItem,
                    listProps: _this.props,
                    onFocusItem: _this.onFocusItem,
                    excludeTabStop: tabbableIndex !== rowIndex
                };
                if (props.item) {
                    return renderRow(rowIndex, props.item, itemDetails);
                }
                else if (renderLoadingRow) {
                    return renderLoadingRow(rowIndex, itemDetails);
                }
                else {
                    return _this.renderLoadingRow(rowIndex, itemDetails);
                }
            }));
            // Save the row in our cache.
            this.state.renderedRows[rowIndex] = renderedRow;
        }
        return renderedRow;
    };
    List.prototype.renderSpacer = function (key, rowCount) {
        var _this = this;
        return (React.createElement("tr", { className: "bolt-list-row-spacer", key: key, ref: function (spacerElement) {
                var existingElement = _this.spacerElements[key];
                if (spacerElement) {
                    if (existingElement !== spacerElement) {
                        if (existingElement) {
                            _this.intersectionContext.unobserve(spacerElement);
                        }
                        _this.intersectionContext.observe(spacerElement);
                        _this.spacerElements[key] = spacerElement;
                    }
                }
                else if (existingElement) {
                    _this.intersectionContext.unobserve(existingElement);
                    delete _this.spacerElements[key];
                }
            } },
            React.createElement("td", { className: "bolt-list-cell-spacer", colSpan: this.props.columns, style: { height: rowCount * this.state.rowHeight + "px" } })));
    };
    List.prototype.rowActivated = function (event, listRow) {
        this.state.eventDispatch.dispatchEvent(event, listRow, "activate");
        if (this.props.onActivate) {
            this.props.onActivate(event, listRow);
            event.preventDefault();
        }
    };
    List.prototype.rowSelected = function (event, listRow) {
        this.state.eventDispatch.dispatchEvent(event, listRow, "select");
        if (this.props.onSelect) {
            this.props.onSelect(event, listRow);
        }
    };
    List.defaultProps = {
        columns: 1,
        defaultTabbableRow: 0,
        focuszoneProps: { direction: FocusZoneDirection.Vertical },
        initialPageCount: 3,
        pageSize: 10,
        role: "listbox",
        singleClickActivation: false
    };
    return List;
}(React.Component));
export { List };
var ScrollableList = /** @class */ (function (_super) {
    tslib_1.__extends(ScrollableList, _super);
    function ScrollableList() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.list = React.createRef();
        return _this;
    }
    ScrollableList.prototype.render = function () {
        return (React.createElement(Intersection, { rootMargin: window.innerHeight / 2 },
            React.createElement("div", { className: "flex-grow scroll-auto" },
                React.createElement(List, tslib_1.__assign({}, this.props, { ref: this.list })))));
    };
    ScrollableList.prototype.addOverlay = function (id, rowIndex, render, zIndex) {
        if (zIndex === void 0) { zIndex = 0; }
        if (this.list.current) {
            return this.list.current.addOverlay(id, rowIndex, render, zIndex);
        }
    };
    ScrollableList.prototype.removeOverlay = function (id) {
        if (this.list.current) {
            return this.list.current.removeOverlay(id);
        }
    };
    ScrollableList.prototype.getFocusIndex = function () {
        if (this.list.current) {
            return this.list.current.getFocusIndex();
        }
        return -1;
    };
    return ScrollableList;
}(React.Component));
export { ScrollableList };
var SimpleList = /** @class */ (function (_super) {
    tslib_1.__extends(SimpleList, _super);
    function SimpleList() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.renderListItem = function (rowIndex, listCell, details) {
            return renderListItem(rowIndex, details, renderListCell(listCell));
        };
        return _this;
    }
    SimpleList.prototype.render = function () {
        var listProps = {
            className: this.props.className,
            columns: 1,
            eventDispatch: this.props.eventDispatch,
            focuszoneProps: this.props.focuszoneProps,
            id: this.props.id,
            initialPageCount: this.props.initialPageCount,
            itemProvider: this.props.itemProvider,
            onActivate: this.props.onActivate,
            renderRow: this.renderListItem,
            onSelect: this.props.onSelect,
            selection: this.props.selection,
            width: this.props.width
        };
        if (this.props.scrollable) {
            return React.createElement(ScrollableList, tslib_1.__assign({}, listProps));
        }
        else {
            return React.createElement(List, tslib_1.__assign({}, listProps));
        }
    };
    return SimpleList;
}(React.Component));
export { SimpleList };
export function renderListItem(rowIndex, details, children) {
    return (React.createElement(ListItem, { details: details, index: rowIndex }, children));
}
var ListItem = /** @class */ (function (_super) {
    tslib_1.__extends(ListItem, _super);
    function ListItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.onFocus = function (event) {
            _this.props.details.onFocusItem(_this.props.index, event);
        };
        return _this;
    }
    ListItem.prototype.render = function () {
        var _this = this;
        var _a = this.props, children = _a.children, details = _a.details, index = _a.index, linkProps = _a.linkProps;
        var ariaBusy = details.ariaBusy, ariaLabel = details.ariaLabel, ariaPosInSet = details.ariaPosInSet, ariaSetSize = details.ariaSetSize, data = details.data, _b = details.excludeTabStop, excludeTabStop = _b === void 0 ? true : _b, excludeFocusZone = details.excludeFocusZone;
        var _c = details.listProps, selection = _c.selection, singleClickActivation = _c.singleClickActivation;
        // If the row is being rendered as a link we will use an anchor, otherwise we will
        // use a standard table row.
        var RowType = linkProps ? "a" : "tr";
        var CellType = linkProps ? "div" : "td";
        // Build the set of props needed from the link to forward on to the row element.
        var linkForwardProps;
        if (linkProps) {
            linkForwardProps = {
                disabled: linkProps.disabled,
                href: linkProps.href,
                id: linkProps.id,
                rel: linkProps.rel,
                role: linkProps.role,
                target: linkProps.target
            };
        }
        return (React.createElement(FocusWithin, { onFocus: this.onFocus }, function (focusStatus) { return (React.createElement(FocusZoneContext.Consumer, null, function (rowContext) {
            return (React.createElement(FocusZone, { direction: FocusZoneDirection.Horizontal },
                React.createElement(RowType, tslib_1.__assign({}, linkForwardProps, { "aria-busy": ariaBusy, "aria-label": ariaLabel, "aria-posinset": ariaPosInSet === undefined ? index + 1 : ariaPosInSet === null ? undefined : ariaPosInSet, "aria-selected": selection && selection.selected(index), "aria-setsize": ariaSetSize === undefined
                        ? _this.props.details.listProps.itemProvider.length
                        : ariaSetSize === null
                            ? undefined
                            : ariaSetSize, className: css(_this.props.className, "bolt-list-row", index === 0 && "first-row", linkProps && "bolt-link", selection && selection.selected(index) && "selected", focusStatus.hasFocus && "focused", singleClickActivation && "single-click-activation"), "data-focuszone": excludeFocusZone ? undefined : rowContext.focuszoneId, "data-row-index": index, tabIndex: excludeTabStop ? -1 : 0, onBlur: focusStatus.onBlur, onFocus: focusStatus.onFocus, role: "listitem" }),
                    React.createElement(CellType, { className: "bolt-list-cell", "data-column-index": 0 },
                        React.createElement("div", { className: "bolt-list-cell-content flex-row" }, children)))));
        })); }));
    };
    return ListItem;
}(React.Component));
export { ListItem };
export function renderListCell(listCell) {
    var textClassName = undefined;
    var textContent = (React.createElement(Tooltip, { overflowOnly: true },
        React.createElement("span", { className: "text-ellipsis" }, typeof listCell === "string" || typeof listCell === "number" ? listCell : listCell.text)));
    var content = textContent;
    var classNames = css("bolt-list-cell-child flex-row flex-center");
    if (typeof listCell !== "string" && typeof listCell !== "number") {
        textClassName = listCell.textClassName;
        if (listCell.iconProps) {
            content = (React.createElement(React.Fragment, null,
                Icon(tslib_1.__assign({}, listCell.iconProps, { className: css("bolt-list-cell-child", listCell.iconProps.className) })),
                textContent));
        }
        if (listCell.href) {
            return (React.createElement(Link, { className: css(textClassName, classNames, "bolt-list-cell-link scroll-hidden"), href: listCell.href, excludeTabStop: true }, content));
        }
    }
    return React.createElement("span", { className: css(textClassName, classNames, "bolt-list-cell-text") }, content);
}
function getAttributeAsNumber(element, attributeName) {
    var attributeValue = element.getAttribute(attributeName);
    if (attributeValue) {
        return parseInt(attributeValue, 10);
    }
    return -1;
}
export function cellFromEvent(event) {
    var attributeValue;
    var cellIndex = -1;
    var rowIndex = -1;
    var element = event.target;
    var cellElement = null;
    while (element) {
        attributeValue = getAttributeAsNumber(element, "data-column-index");
        if (attributeValue !== -1) {
            cellIndex = attributeValue;
            cellElement = element;
        }
        attributeValue = getAttributeAsNumber(element, "data-row-index");
        if (attributeValue !== -1) {
            rowIndex = attributeValue;
            break;
        }
        // We have hit the root of the details list, dont look above this.
        if (element.classList.contains("bolt-list")) {
            element = null;
            break;
        }
        element = element.parentElement;
    }
    return {
        cellElement: cellElement,
        cellIndex: cellIndex,
        rowElement: element,
        rowIndex: rowIndex
    };
}
