import { ObservableArray } from '../Core/Observable';
/**
 * A TreeItemProvider is designed to store and manage the state of a tree.
 * As items are added/removed, expanded/collapsed, the providers "value" will
 * represent the current set of ordered items within the tree.
 */
var TreeItemProvider = /** @class */ (function () {
    function TreeItemProvider(rootItems) {
        var initialItems = [];
        // Make a copy of the initial root items
        this.rootItems = rootItems.slice();
        // Initialize the ObservableArray with the initial item state.
        for (var index = 0; index < rootItems.length; index++) {
            var item = rootItems[index];
            this.getExpandedSubTree(item, initialItems);
        }
        // Create the underlying observable we use to manage the items
        this.internalItems = new ObservableArray(initialItems);
    }
    Object.defineProperty(TreeItemProvider.prototype, "length", {
        get: function () {
            return this.internalItems.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TreeItemProvider.prototype, "value", {
        get: function () {
            return this.internalItems.value;
        },
        enumerable: true,
        configurable: true
    });
    TreeItemProvider.prototype.subscribe = function (observer, action) {
        this.internalItems.subscribe(observer, action);
    };
    TreeItemProvider.prototype.unsubscribe = function (observer, action) {
        return this.internalItems.unsubscribe(observer, action);
    };
    /**
     * add can be used to add a single item to the tree at a given location. If the
     * specified parentItem or insertAfter are not in the tree or the insertAfter is not
     * a child of parentItem, the addition will be ignored.
     *
     * @param item The item to add to the tree.
     * @param parentItem The direct parent of the item being added.
     * If this is a rootItem leave the parentItem undefined.
     * @param insertAfter The sibling item this item should be inserted after.
     * If this is the first item within the parent leave insertAfter undefined.
     */
    TreeItemProvider.prototype.add = function (item, parentItem, insertAfter) {
        return this.splice(parentItem, undefined, [
            {
                insertAfter: insertAfter,
                items: [item]
            }
        ]);
    };
    /**
     * Remove the specified item from the tree. If the item doesnt exist the
     * remove will be ignored.
     *
     * @param treeItem The item that should be removed from the tree.
     * @param parentItem The parentItem of the item being removed. If the caller
     * doesnt have the parentItem readily available it will be computed, but this
     * can be expensive. The caller should supply it if they can.
     */
    TreeItemProvider.prototype.remove = function (treeItem, parentItem) {
        if (!parentItem) {
            var itemIndex = this.indexOf(treeItem);
            if ((itemIndex = this.indexOf(treeItem)) === -1) {
                return;
            }
            var removeItem = this.internalItems.value[itemIndex];
            parentItem = removeItem.parentItem && removeItem.parentItem.underlyingItem;
        }
        return this.splice(parentItem, [treeItem]);
    };
    /**
     * spliceItems is used to update the direct children of a given item
     * in the tree. If the parentItem is supplied and not in the tree the
     * changes will be ignored.
     *
     * @NOTE: Adds are processed before removes, this allows the existing
     * elements to be used as an insertAfter for placement. An item can't
     * be in both the add and remove set. This will cause duplication.
     *
     * @param parentItem The item whos children are being updated. If undefined
     * is supplied the updates will modify the root.
     * @param itemsToRemove The set of items that should removed from the parent.
     * @param itemsToAdd The set of items that should be added to the parent.
     */
    TreeItemProvider.prototype.splice = function (parentItem, itemsToRemove, itemsToAdd) {
        var _a;
        var parentItemEx;
        var notifyChange = false;
        var childItems = this.rootItems;
        var parentIndex = -1;
        // If a parent was supplied get its children, this is the collection we
        // will be modifying.
        if (parentItem) {
            // If the parent can't be found, we will ignore the splice operation.
            if ((parentIndex = this.indexOf(parentItem)) === -1) {
                return;
            }
            parentItemEx = this.internalItems.value[parentIndex];
            childItems = parentItemEx.underlyingItem.childItems || [];
        }
        // Before making any changes make a copy of the removed array, this allows
        // the caller to supply the current childItems to clear them.
        if (itemsToRemove) {
            itemsToRemove = itemsToRemove.slice(0);
        }
        // We will process the adds before the removes.
        if (itemsToAdd) {
            for (var index = 0; index < itemsToAdd.length; index++) {
                var itemToAdd = itemsToAdd[index];
                var insertIndex = parentIndex;
                var childIndex = 0;
                var childCount = 0;
                // If we are inserting after a specific item find that item. If it
                // isn't found, ignore the add.
                if (itemToAdd.insertAfter) {
                    // Ensure the insertAfter is a child of the parentItem.
                    if ((childIndex = childItems.indexOf(itemToAdd.insertAfter)) === -1) {
                        continue;
                    }
                    // Find the location in the underlying observable to insert.
                    insertIndex = this.indexOf(itemToAdd.insertAfter, parentIndex + 1);
                    if (insertIndex === -1) {
                        continue;
                    }
                    // We need to add after any of this childs visible children.
                    childCount = this.getChildCount(itemToAdd.insertAfter);
                }
                // Update the childItems and if this is the first child update the treeItem.
                childItems.splice.apply(childItems, [childIndex, 0].concat(itemToAdd.items));
                if (parentItemEx && !parentItemEx.underlyingItem.childItems) {
                    parentItemEx.underlyingItem.childItems = childItems;
                }
                // Add the new items to the underlying observable if the parent is expanded.
                if (!parentItem || parentItem.expanded) {
                    // Build the set of expanded items being added.
                    var addedItems = [];
                    for (var childIndex_1 = 0; childIndex_1 < itemToAdd.items.length; childIndex_1++) {
                        this.getExpandedSubTree(itemToAdd.items[childIndex_1], addedItems, parentItemEx);
                    }
                    (_a = this.internalItems).splice.apply(_a, [insertIndex + childCount + 1, 0].concat(addedItems));
                    notifyChange = true;
                }
            }
        }
        // Now process the removes.
        if (itemsToRemove) {
            for (var index = 0; index < itemsToRemove.length; index++) {
                var itemToRemove = itemsToRemove[index];
                var childIndex = void 0;
                // Ensure the itemToRemove is a child of the parentItem.
                if ((childIndex = childItems.indexOf(itemToRemove)) === -1) {
                    continue;
                }
                // Remove the item from this childItem list.
                childItems.splice(childIndex, 1);
                if (childItems.length === 0 && parentItem) {
                    delete parentItem.childItems;
                }
                var removeIndex = this.indexOf(itemToRemove, parentIndex + 1);
                if (removeIndex === -1) {
                    continue;
                }
                // Remove the items from the underlying observable.
                var existingItem = this.internalItems.value[removeIndex].underlyingItem;
                this.internalItems.splice(removeIndex, this.getChildCount(existingItem) + 1);
                notifyChange = true;
            }
        }
        if (notifyChange && parentItemEx) {
            this.internalItems.change(parentIndex, parentItemEx);
        }
    };
    /**
     * toggleItem is used to toggle the expand/collapse state of a given tree item.
     *
     * @param treeItem The item whose state should be toggled. If the treeItem is
     * not in the tree the method will no-op.
     */
    TreeItemProvider.prototype.toggle = function (treeItem) {
        var _a;
        var itemIndex = this.indexOf(treeItem);
        if (itemIndex >= 0) {
            var treeItemEx = this.internalItems.value[itemIndex];
            // Toggle the expanded state of the treeItem.
            treeItem.expanded = !treeItem.expanded;
            if (treeItem.childItems) {
                var childItems = [];
                // Get the set of children being added to the underlying array.
                for (var index = 0; index < treeItem.childItems.length; index++) {
                    var item = treeItem.childItems[index];
                    this.getExpandedSubTree(item, childItems, treeItemEx);
                }
                // We need to update the underlying observables items.
                if (treeItem.expanded) {
                    (_a = this.internalItems).splice.apply(_a, [itemIndex + 1, 0].concat(childItems));
                }
                else {
                    this.internalItems.splice(itemIndex + 1, childItems.length);
                }
            }
        }
    };
    TreeItemProvider.prototype.getChildCount = function (treeItem) {
        var count = 0;
        if (treeItem.childItems && treeItem.expanded) {
            for (var index = 0; index < treeItem.childItems.length; index++) {
                count += this.getChildCount(treeItem.childItems[index]) + 1;
            }
        }
        return count;
    };
    TreeItemProvider.prototype.getExpandedSubTree = function (treeItem, items, parentItem) {
        var currentItem = { depth: parentItem ? parentItem.depth + 1 : 0, parentItem: parentItem, underlyingItem: treeItem };
        // Save the current item at the end of the set of items. Use the internal
        // array we dont want to notify on every insertion, that caller will send
        // the appropriate notification event.
        items.push(currentItem);
        // Expand the children of this item after this after.
        if (treeItem.childItems && treeItem.expanded) {
            for (var _i = 0, _a = treeItem.childItems; _i < _a.length; _i++) {
                var childItem = _a[_i];
                this.getExpandedSubTree(childItem, items, currentItem);
            }
        }
    };
    /**
     * indexOfItem is used to find the index of a source treeItem. The underlying
     * observable indexOf will find instances of ITreeItemEx. This returns the
     * index in the table which is only visible rows.
     *
     * @param treeItem The item to find in the tree.
     * @param fromIndex The index to start the search.
     * @returns If the item is found an index >= 0 is returned, if it is not found -1 is returned.
     */
    TreeItemProvider.prototype.indexOf = function (treeItem, fromIndex) {
        if (fromIndex === void 0) { fromIndex = 0; }
        // @TODO: Can we come up with a faster method than this.
        for (var index = fromIndex; index < this.internalItems.length; index++) {
            if (treeItem === this.internalItems.value[index].underlyingItem) {
                return index;
            }
        }
        return -1;
    };
    return TreeItemProvider;
}());
export { TreeItemProvider };
